@@ -58,7 +58,7 @@ except ImportError:
 
 from bigml.util import (invert_dictionary, localize, is_url, check_dir,
                         clear_console_line, reset_console_line, console_log,
-                        maybe_save)
+                        maybe_save, get_exponential_wait)
 from bigml.util import DEFAULT_LOCALE
 
 register_openers()
@@ -169,6 +169,9 @@ STATUSES = {
     RUNNABLE: "RUNNABLE"
 }
 
+# Minimum query string to get model fields
+TINY_MODEL = "full=false"
+
 
 def get_resource(regex, resource):
     """Returns a resource/id.
@@ -278,7 +281,7 @@ def resource_is_ready(resource):
         raise Exception("No valid resource structure found")
     if resource['error'] is not None:
         raise Exception(resource['error']['status']['message'])
-    return (resource['code'] == HTTP_OK and
+    return (resource['code'] in [HTTP_OK, HTTP_ACCEPTED] and
             get_status(resource)['code'] == FINISHED)
 
 
@@ -300,7 +303,7 @@ def get_status(resource):
 
 
 def check_resource(resource, get_method, query_string='', wait_time=1,
-                   retries=None):
+                   retries=None, raise_on_error=False):
     """Waits until a resource is finished.
 
        Given a resource and its corresponding get_method
@@ -342,14 +345,26 @@ def check_resource(resource, get_method, query_string='', wait_time=1,
         status = get_status(resource)
         code = status['code']
         if code == FINISHED:
+            if raise_on_error:
+                exception_on_error(resource)
             return resource
         elif code == FAULTY:
             raise ValueError(status)
-        time.sleep(wait_time)
+        time.sleep(get_exponential_wait(wait_time, counter))
         resource = get_method(resource, **kwargs)
+    if raise_on_error:
+        exception_on_error(resource)
     return resource
 
 
+def exception_on_error(resource):
+    """Raises exception if resource has error
+
+    """
+    if resource['error'] is not None:
+        raise Exception(resource['error']['status']['message'])
+
+
 def error_message(resource, resource_type='resource', method=None):
     """Error message for each type of resource
 
@@ -896,13 +911,14 @@ class BigML(object):
             origin_datasets = [datasets]
         else:
             origin_datasets = datasets
+
         for dataset in origin_datasets:
             check_resource_type(dataset, DATASET_PATH,
                                 message="A dataset id is needed to create a"
                                         " model.")
             dataset = check_resource(dataset, self.get_dataset,
-                                     wait_time=wait_time, retries=retries)
-
+                                     wait_time=wait_time, retries=retries,
+                                     raise_on_error=True)
             dataset_ids.append(get_dataset_id(dataset))
 
         create_args = {}
@@ -949,23 +965,38 @@ class BigML(object):
         the resource keyed by Id.
 
         """
+
+        def _get_fields_key(resource):
+            """Returns the fields key from a resource dict
+
+            """
+            if resource['code'] in [HTTP_OK, HTTP_ACCEPTED]:
+                if MODEL_RE.match(resource_id):
+                    return resource['object']['model']['model_fields']
+                else:
+                    return resource['object']['fields']
+            return None
+
         if isinstance(resource, dict) and 'resource' in resource:
             resource_id = resource['resource']
         elif (isinstance(resource, basestring) and (
               SOURCE_RE.match(resource) or DATASET_RE.match(resource) or
               MODEL_RE.match(resource) or PREDICTION_RE.match(resource))):
             resource_id = resource
+            resource = self._get("%s%s" % (self.url, resource_id))
         else:
             LOGGER.error("Wrong resource id")
             return
+        # Tries to extract fields information from resource dict. If it fails,
+        # a get remote call is used to retrieve the resource by id.
+        fields = None
+        try:
+            fields = _get_fields_key(resource)
+        except KeyError:
+            resource = self._get("%s%s" % (self.url, resource_id))
+            fields = _get_fields_key(resource)
 
-        resource = self._get("%s%s" % (self.url, resource_id))
-        if resource['code'] == HTTP_OK:
-            if MODEL_RE.match(resource_id):
-                return resource['object']['model']['model_fields']
-            else:
-                return resource['object']['fields']
-        return None
+        return fields
 
     def pprint(self, resource, out=sys.stdout):
         """Pretty prints a resource or part of it.
@@ -1027,6 +1058,52 @@ class BigML(object):
         return check_resource(resource, get_method,
                               query_string=query_string, wait_time=wait_time)
 
+    def check_origin_dme(self, dataset, model_or_ensemble, args, wait_time=3,
+                         retries=10):
+        """Returns True if the dataset, model or ensemble needed to build
+           the batch prediction or evaluation are finished. The args given
+           by the user are modified to include the related ids in the
+           create call.
+
+        """
+
+        def args_update(get_method):
+            """Updates args when the resource is ready
+
+            """
+            if resource_id:
+                resource = check_resource(resource_id, get_method,
+                                          wait_time=wait_time, retries=retries,
+                                          raise_on_error=True)
+                args.update({
+                    resource_type: resource_id,
+                    "dataset": dataset_id})
+
+        resource_type = get_resource_type(dataset)
+        if not DATASET_PATH == resource_type:
+            raise Exception("A dataset id is needed as second argument"
+                            " to create the resource. %s found." %
+                            resource_type)
+        dataset_id = get_dataset_id(dataset)
+        if dataset_id:
+            dataset = check_resource(dataset_id, self.get_dataset,
+                                     wait_time=wait_time, retries=retries,
+                                     raise_on_error=True)
+            resource_type = get_resource_type(model_or_ensemble)
+            if resource_type == MODEL_PATH:
+                resource_id = get_model_id(model_or_ensemble)
+                args_update(self.get_model)
+            elif resource_type == ENSEMBLE_PATH:
+                resource_id = get_ensemble_id(model_or_ensemble)
+                args_update(self.get_ensemble)
+            else:
+                raise Exception("A model or ensemble id is needed as first"
+                                " argument to create the resource."
+                                " %s found." % resource_type)
+
+        return dataset_id and resource_id
+
+
     ##########################################################################
     #
     # Sources
@@ -1079,7 +1156,6 @@ class BigML(object):
                                      data=create_args, verify=VERIFY)
 
             code = response.status_code
-
             if code == HTTP_CREATED:
                 location = response.headers['location']
                 resource = json.loads(response.content, 'utf-8')
@@ -1225,7 +1301,6 @@ class BigML(object):
         except ValueError:
             LOGGER.error("Malformed response")
         except urllib2.HTTPError, exception:
-            LOGGER.error("Error %s", exception.code)
             code = exception.code
             if code in [HTTP_BAD_REQUEST,
                         HTTP_UNAUTHORIZED,
@@ -1234,6 +1309,7 @@ class BigML(object):
                         HTTP_TOO_MANY_REQUESTS]:
                 content = exception.read()
                 error = json.loads(content, 'utf-8')
+                LOGGER.error(error_message(error, method='create'))
             else:
                 LOGGER.error("Unexpected error (%s)" % code)
                 code = HTTP_INTERNAL_SERVER_ERROR
@@ -1350,12 +1426,10 @@ class BigML(object):
             if resource_type == SOURCE_PATH:
                 source_id = get_source_id(source_or_datasets)
                 if source_id:
-                    if wait_time > 0:
-                        fcounter = 0
-                        while (not self.source_is_ready(source_id) and
-                               fcounter < retries):
-                            time.sleep(wait_time)
-                            fcounter += 1
+                    source = check_resource(source_id, self.get_source,
+                                            wait_time=wait_time,
+                                            retries=retries,
+                                            raise_on_error=True)
                     create_args.update({
                         "source": source_id})
             elif resource_type == DATASET_PATH:
@@ -1423,6 +1497,32 @@ class BigML(object):
         if dataset_id:
             return self._delete("%s%s" % (self.url, dataset_id))
 
+    def error_counts(self, dataset, raise_on_error=True):
+        """Returns the ids of the fields that contain errors and their number.
+
+           The dataset argument can be either a dataset resource structure
+           or a dataset id (that will be used to retrieve the associated
+           remote resource).
+
+        """
+        errors_dict = {}
+        if not isinstance(dataset, dict) or not 'object' in dataset: 
+            check_resource_type(dataset, DATASET_PATH,
+                                message="A dataset id is needed.")
+            dataset_id = get_dataset_id(dataset)
+            dataset = check_resource(dataset_id, self.get_dataset,
+                                     raise_on_error=raise_on_error)
+            if not raise_on_error and resource['error'] is not None:
+                dataset_id = None
+        else:
+            dataset_id = get_dataset_id(dataset)
+        if dataset_id:
+            errors = dataset.get('object', {}).get(
+                'status', {}).get('field_errors', {})
+            for field_id in errors:
+                errors_dict[field_id] = errors[field_id]['total']
+        return errors_dict
+
     ##########################################################################
     #
     # Models
@@ -1505,8 +1605,9 @@ class BigML(object):
     #
     ##########################################################################
     def create_prediction(self, model_or_ensemble, input_data=None,
-                          by_name=True, args=None, wait_time=3, retries=10):
+                          args=None, wait_time=3, retries=10, by_name=True):
         """Creates a new prediction.
+           The by_name argument is now deprecated. It will be removed.
 
         """
         ensemble_id = None
@@ -1516,64 +1617,36 @@ class BigML(object):
         if resource_type == ENSEMBLE_PATH:
             ensemble_id = get_ensemble_id(model_or_ensemble)
             if ensemble_id is not None:
-                if wait_time > 0:
-                    counter = 0
-                    while (not self.ensemble_is_ready(ensemble_id) and
-                           counter < retries):
-                        time.sleep(wait_time)
-                        counter += 1
-                try:
-                    ensemble = self.get_ensemble(ensemble_id)
-                    model_id = ensemble['object']['models'][0]
-                except (KeyError, IndexError), exc:
-                    LOGGER.error("The ensemble has no valid model"
-                                 " information: %s" % str(exc))
-                    model_id = None
+                ensemble = check_resource(ensemble_id, self.get_ensemble,
+                                          wait_time=wait_time, retries=retries,
+                                          raise_on_error=True)
         elif resource_type == MODEL_PATH:
             model_id = get_model_id(model_or_ensemble)
+            model = check_resource(model_id, self.get_model,
+                                   query_string=TINY_MODEL,
+                                   wait_time=wait_time, retries=retries,
+                                   raise_on_error=True)
         else:
             raise Exception("A model or ensemble id is needed to create a"
                             " prediction. %s found." % resource_type)
 
-        if model_id is not None:
-            if ensemble_id is None:
-                if wait_time > 0:
-                    counter = 0
-                    while (not self.model_is_ready(model_id) and
-                           counter < retries):
-                        time.sleep(wait_time)
-                        counter += 1
-
-            if input_data is None:
-                input_data = {}
-            elif by_name:
-                fields = self.get_fields(model_id)
-                inverted_fields = invert_dictionary(fields)
-                wrong_keys = [key for key in input_data.keys() if not key
-                              in inverted_fields]
-                if wrong_keys:
-                    LOGGER.info(("Some input fields are"
-                                 " not used in the model: %s") %
-                                ", ".join(wrong_keys))
-                input_data = dict(
-                    [[inverted_fields[key], value]
-                     for key, value in input_data.items()
-                     if key in inverted_fields])
-            create_args = {}
-            if args is not None:
-                create_args.update(args)
+        if input_data is None:
+            input_data = {}
+        create_args = {}
+        if args is not None:
+            create_args.update(args)
+        create_args.update({
+            "input_data": input_data})
+        if ensemble_id is None:
             create_args.update({
-                "input_data": input_data})
-            if ensemble_id is None:
-                create_args.update({
-                    "model": model_id})
-            else:
-                create_args.update({
-                    "ensemble": ensemble_id})
+                "model": model_id})
+        else:
+            create_args.update({
+                "ensemble": ensemble_id})
 
-            body = json.dumps(create_args)
-            return self._create(self.prediction_url, body,
-                                verify=VERIFY_PREDICTION_SERVER)
+        body = json.dumps(create_args)
+        return self._create(self.prediction_url, body,
+                            verify=VERIFY_PREDICTION_SERVER)
 
     def get_prediction(self, prediction):
         """Retrieves a prediction.
@@ -1627,49 +1700,13 @@ class BigML(object):
         if args is not None:
             create_args.update(args)
 
-        def args_update(check_resource_is_ready):
-            """Updates args when the resource is ready
+        origin_resources_checked = self.check_origin_dme(
+            dataset, model_or_ensemble, create_args,
+            wait_time=wait_time, retries=retries)
 
-            """
-            if resource_id:
-                if wait_time > 0:
-                    counter = 0
-                    while (not check_resource_is_ready(resource_id) and
-                           counter < retries):
-                        time.sleep(wait_time)
-                        counter += 1
-                create_args.update({
-                    resource_type: resource_id,
-                    "dataset": dataset_id})
-
-        resource_type = get_resource_type(dataset)
-        if not DATASET_PATH == resource_type:
-            raise Exception("A dataset id is needed as second argument"
-                            " to create an evaluation. %s found." %
-                            resource_type)
-        dataset_id = get_dataset_id(dataset)
-        if dataset_id:
-            if wait_time > 0:
-                counter = 0
-                while (not self.dataset_is_ready(dataset_id) and
-                       counter < retries):
-                    time.sleep(wait_time)
-                    counter += 1
-
-        resource_type = get_resource_type(model_or_ensemble)
-        if resource_type == MODEL_PATH:
-            resource_id = get_model_id(model_or_ensemble)
-            args_update(self.model_is_ready)
-        elif resource_type == ENSEMBLE_PATH:
-            resource_id = get_ensemble_id(model_or_ensemble)
-            args_update(self.ensemble_is_ready)
-        else:
-            raise Exception("A model or ensemble id is needed as first"
-                            " argument to create an"
-                            " evaluation. %s found." % resource_type)
-
-        body = json.dumps(create_args)
-        return self._create(self.evaluation_url, body)
+        if origin_resources_checked:
+            body = json.dumps(create_args)
+            return self._create(self.evaluation_url, body)
 
     def get_evaluation(self, evaluation):
         """Retrieves an evaluation.
@@ -1799,49 +1836,13 @@ class BigML(object):
         if args is not None:
             create_args.update(args)
 
-        def args_update(check_resource_is_ready):
-            """Updates args when the resource is ready
-
-            """
-            if resource_id:
-                if wait_time > 0:
-                    counter = 0
-                    while (not check_resource_is_ready(resource_id) and
-                           counter < retries):
-                        time.sleep(wait_time)
-                        counter += 1
-                create_args.update({
-                    resource_type: resource_id,
-                    "dataset": dataset_id})
-
-        resource_type = get_resource_type(dataset)
-        if not DATASET_PATH == resource_type:
-            raise Exception("A dataset id is needed as second argument"
-                            " to create a batch prediction. %s found." %
-                            resource_type)
-        dataset_id = get_dataset_id(dataset)
-        if dataset_id:
-            if wait_time > 0:
-                counter = 0
-                while (not self.dataset_is_ready(dataset_id) and
-                       counter < retries):
-                    time.sleep(wait_time)
-                    counter += 1
-
-        resource_type = get_resource_type(model_or_ensemble)
-        if resource_type == MODEL_PATH:
-            resource_id = get_model_id(model_or_ensemble)
-            args_update(self.model_is_ready)
-        elif resource_type == ENSEMBLE_PATH:
-            resource_id = get_ensemble_id(model_or_ensemble)
-            args_update(self.ensemble_is_ready)
-        else:
-            raise Exception("A model or ensemble id is needed as first"
-                            " argument to create a"
-                            " batch prediction. %s found." % resource_type)
-
-        body = json.dumps(create_args)
-        return self._create(self.batch_prediction_url, body)
+        origin_resources_checked = self.check_origin_dme(
+            dataset, model_or_ensemble, create_args,
+            wait_time=wait_time, retries=retries)
+                    
+        if origin_resources_checked:
+            body = json.dumps(create_args)
+            return self._create(self.batch_prediction_url, body)
 
     def get_batch_prediction(self, batch_prediction):
         """Retrieves a batch prediction.
